@@ -62,11 +62,36 @@ if PY3:
 else:
     import imp as importlib
 
+__all__ = [
+    # constants
+    'APPVEYOR', 'DEVNULL', 'GLOBAL_TIMEOUT', 'MEMORY_TOLERANCE', 'NO_RETRIES',
+    'PYPY', 'PYTHON', 'RLIMIT_SUPPORT', 'ROOT_DIR', 'SCRIPTS_DIR',
+    'TESTFILE_PREFIX', 'TESTFN', 'TESTFN_UNICODE', 'TOX', 'TRAVIS',
+    'VALID_PROC_STATUSES', 'VERBOSITY',
+    # classes
+    'ThreadTask'
+    # test utils
+    'check_connection_ntuple', 'check_net_address', 'unittest', 'cleanup',
+    'skip_on_access_denied', 'skip_on_not_implemented', 'retry_before_failing',
+    'run_test_module_by_name',
+    # fs utils
+    'chdir', 'safe_remove', 'safe_rmdir', 'create_temp_executable_file',
+    # subprocesses
+    'pyrun', 'reap_children', 'get_test_subprocess',
+    # os
+    'get_winver', 'get_kernel_version',
+    # sync primitives
+    'call_until', 'wait_for_pid', 'wait_for_file',
+    # others
+    'warn', 'decode_path', 'encode_path',
+]
+
 
 # ===================================================================
-# --- Constants
+# --- constants
 # ===================================================================
 
+
 # conf for retry_before_failing() decorator
 NO_RETRIES = 10
 # bytes tolerance for OS memory related tests
@@ -110,9 +135,10 @@ VERBOSITY = 1 if os.getenv('SILENT') or TOX else 2
 
 
 # ===================================================================
-# --- Classes
+# --- classes
 # ===================================================================
 
+
 class ThreadTask(threading.Thread):
     """A thread object used for running process thread tests."""
 
@@ -151,22 +177,9 @@ class ThreadTask(threading.Thread):
 
 
 # ===================================================================
-# --- Utility functions
+# --- subprocesses
 # ===================================================================
 
-def cleanup():
-    reap_children(search_all=True)
-    safe_remove(TESTFN)
-    try:
-        safe_rmdir(TESTFN_UNICODE)
-    except UnicodeEncodeError:
-        pass
-    for path in _testfiles:
-        safe_remove(path)
-
-atexit.register(cleanup)
-atexit.register(lambda: DEVNULL.close())
-
 
 _subprocesses_started = set()
 
@@ -228,11 +241,6 @@ def pyrun(src):
         return subp
 
 
-def warn(msg):
-    """Raise a warning msg."""
-    warnings.warn(msg, UserWarning)
-
-
 def sh(cmdline, stdout=subprocess.PIPE, stderr=subprocess.PIPE):
     """run cmd in a subprocess and return its output.
     raises RuntimeError on error.
@@ -252,70 +260,6 @@ def sh(cmdline, stdout=subprocess.PIPE, stderr=subprocess.PIPE):
     return stdout.strip()
 
 
-if POSIX:
-    def get_kernel_version():
-        """Return a tuple such as (2, 6, 36)."""
-        s = ""
-        uname = os.uname()[2]
-        for c in uname:
-            if c.isdigit() or c == '.':
-                s += c
-            else:
-                break
-        if not s:
-            raise ValueError("can't parse %r" % uname)
-        minor = 0
-        micro = 0
-        nums = s.split('.')
-        major = int(nums[0])
-        if len(nums) >= 2:
-            minor = int(nums[1])
-        if len(nums) >= 3:
-            micro = int(nums[2])
-        return (major, minor, micro)
-else:
-    def get_kernel_version():
-        return ()
-
-
-if LINUX:
-    RLIMIT_SUPPORT = get_kernel_version() >= (2, 6, 36)
-else:
-    RLIMIT_SUPPORT = False
-
-
-def wait_for_pid(pid, timeout=GLOBAL_TIMEOUT):
-    """Wait for pid to show up in the process list then return.
-    Used in the test suite to give time the sub process to initialize.
-    """
-    raise_at = time.time() + timeout
-    while True:
-        if pid in psutil.pids():
-            # give it one more iteration to allow full initialization
-            time.sleep(0.01)
-            return
-        time.sleep(0.0001)
-        if time.time() >= raise_at:
-            raise RuntimeError("Timed out")
-
-
-def wait_for_file(fname, timeout=GLOBAL_TIMEOUT, delete_file=True):
-    """Wait for a file to be written on disk."""
-    stop_at = time.time() + 3
-    while time.time() < stop_at:
-        try:
-            with open(fname, "r") as f:
-                data = f.read()
-            if not data:
-                continue
-            if delete_file:
-                os.remove(fname)
-            return data
-        except IOError:
-            time.sleep(0.001)
-    raise RuntimeError("timed out (couldn't read file)")
-
-
 def reap_children(search_all=False):
     """Kill any subprocess started by this test suite and ensure that
     no zombies stick around to hog resources and create problems when
@@ -345,6 +289,219 @@ def reap_children(search_all=False):
     _subprocesses_started = set(alive)
 
 
+# ===================================================================
+# --- OS
+# ===================================================================
+
+
+if not POSIX:
+    def get_kernel_version():
+        return ()
+else:
+    def get_kernel_version():
+        """Return a tuple such as (2, 6, 36)."""
+        s = ""
+        uname = os.uname()[2]
+        for c in uname:
+            if c.isdigit() or c == '.':
+                s += c
+            else:
+                break
+        if not s:
+            raise ValueError("can't parse %r" % uname)
+        minor = 0
+        micro = 0
+        nums = s.split('.')
+        major = int(nums[0])
+        if len(nums) >= 2:
+            minor = int(nums[1])
+        if len(nums) >= 3:
+            micro = int(nums[2])
+        return (major, minor, micro)
+
+
+if LINUX:
+    RLIMIT_SUPPORT = get_kernel_version() >= (2, 6, 36)
+else:
+    RLIMIT_SUPPORT = False
+
+
+if not WINDOWS:
+    def get_winver():
+        raise NotImplementedError("not a Windows OS")
+else:
+    def get_winver():
+        wv = sys.getwindowsversion()
+        if hasattr(wv, 'service_pack_major'):  # python >= 2.7
+            sp = wv.service_pack_major or 0
+        else:
+            r = re.search("\s\d$", wv[4])
+            if r:
+                sp = int(r.group(0))
+            else:
+                sp = 0
+        return (wv[0], wv[1], sp)
+
+
+# ===================================================================
+# --- sync primitives
+# ===================================================================
+
+
+def wait_for_pid(pid, timeout=GLOBAL_TIMEOUT):
+    """Wait for pid to show up in the process list then return.
+    Used in the test suite to give time the sub process to initialize.
+    """
+    raise_at = time.time() + timeout
+    while True:
+        if pid in psutil.pids():
+            # give it one more iteration to allow full initialization
+            time.sleep(0.01)
+            return
+        time.sleep(0.0001)
+        if time.time() >= raise_at:
+            raise RuntimeError("Timed out")
+
+
+def wait_for_file(fname, timeout=GLOBAL_TIMEOUT, delete_file=True):
+    """Wait for a file to be written on disk."""
+    stop_at = time.time() + timeout
+    while time.time() < stop_at:
+        try:
+            with open(fname, "r") as f:
+                data = f.read()
+            if not data:
+                continue
+            if delete_file:
+                os.remove(fname)
+            return data
+        except IOError:
+            time.sleep(0.001)
+    raise RuntimeError(
+        "timed out after %s secs (couldn't read file)" % timeout)
+
+
+def call_until(fun, expr, timeout=GLOBAL_TIMEOUT):
+    """Keep calling function for timeout secs and exit if eval()
+    expression is True.
+    """
+    stop_at = time.time() + timeout
+    while time.time() < stop_at:
+        ret = fun()
+        if eval(expr):
+            return ret
+        time.sleep(0.001)
+    raise RuntimeError('timed out after %s secs (ret=%r)' % (timeout, ret))
+
+
+# ===================================================================
+# --- fs
+# ===================================================================
+
+
+def safe_remove(file):
+    "Convenience function for removing temporary test files"
+    try:
+        os.remove(file)
+    except OSError as err:
+        if err.errno != errno.ENOENT:
+            # # file is being used by another process
+            # if WINDOWS and isinstance(err, WindowsError) and err.errno == 13:
+            #     return
+            raise
+
+
+def safe_rmdir(dir):
+    "Convenience function for removing temporary test directories"
+    try:
+        os.rmdir(dir)
+    except OSError as err:
+        if err.errno != errno.ENOENT:
+            raise
+
+
+@contextlib.contextmanager
+def chdir(dirname):
+    """Context manager which temporarily changes the current directory."""
+    curdir = os.getcwd()
+    try:
+        os.chdir(dirname)
+        yield
+    finally:
+        os.chdir(curdir)
+
+
+# ===================================================================
+# --- testing
+# ===================================================================
+
+
+def retry_before_failing(ntimes=None):
+    """Decorator which runs a test function and retries N times before
+    actually failing.
+    """
+    def decorator(fun):
+        @functools.wraps(fun)
+        def wrapper(*args, **kwargs):
+            times = ntimes or NO_RETRIES
+            assert times, times
+            for x in range(times):
+                try:
+                    return fun(*args, **kwargs)
+                except AssertionError as _:
+                    err = _
+            raise err
+        return wrapper
+    return decorator
+
+
+def run_test_module_by_name(name):
+    # testmodules = [os.path.splitext(x)[0] for x in os.listdir(HERE)
+    #                if x.endswith('.py') and x.startswith('test_')]
+    name = os.path.splitext(os.path.basename(name))[0]
+    suite = unittest.TestSuite()
+    suite.addTest(unittest.defaultTestLoader.loadTestsFromName(name))
+    result = unittest.TextTestRunner(verbosity=VERBOSITY).run(suite)
+    success = result.wasSuccessful()
+    sys.exit(0 if success else 1)
+
+
+def skip_on_access_denied(only_if=None):
+    """Decorator to Ignore AccessDenied exceptions."""
+    def decorator(fun):
+        @functools.wraps(fun)
+        def wrapper(*args, **kwargs):
+            try:
+                return fun(*args, **kwargs)
+            except psutil.AccessDenied:
+                if only_if is not None:
+                    if not only_if:
+                        raise
+                msg = "%r was skipped because it raised AccessDenied" \
+                      % fun.__name__
+                raise unittest.SkipTest(msg)
+        return wrapper
+    return decorator
+
+
+def skip_on_not_implemented(only_if=None):
+    """Decorator to Ignore NotImplementedError exceptions."""
+    def decorator(fun):
+        @functools.wraps(fun)
+        def wrapper(*args, **kwargs):
+            try:
+                return fun(*args, **kwargs)
+            except NotImplementedError:
+                if only_if is not None:
+                    if not only_if:
+                        raise
+                msg = "%r was skipped because it raised NotImplementedError" \
+                      % fun.__name__
+                raise unittest.SkipTest(msg)
+        return wrapper
+    return decorator
+
+
 def check_net_address(addr, family):
     """Check a net address validity. Supported families are IPv4,
     IPv6 and MAC addresses.
@@ -432,106 +589,6 @@ def check_connection_ntuple(conn):
                     assert dupsock.type == conn.type
 
 
-def safe_remove(file):
-    "Convenience function for removing temporary test files"
-    try:
-        os.remove(file)
-    except OSError as err:
-        if err.errno != errno.ENOENT:
-            # # file is being used by another process
-            # if WINDOWS and isinstance(err, WindowsError) and err.errno == 13:
-            #     return
-            raise
-
-
-def safe_rmdir(dir):
-    "Convenience function for removing temporary test directories"
-    try:
-        os.rmdir(dir)
-    except OSError as err:
-        if err.errno != errno.ENOENT:
-            raise
-
-
-@contextlib.contextmanager
-def chdir(dirname):
-    """Context manager which temporarily changes the current directory."""
-    curdir = os.getcwd()
-    try:
-        os.chdir(dirname)
-        yield
-    finally:
-        os.chdir(curdir)
-
-
-def call_until(fun, expr, timeout=GLOBAL_TIMEOUT):
-    """Keep calling function for timeout secs and exit if eval()
-    expression is True.
-    """
-    stop_at = time.time() + timeout
-    while time.time() < stop_at:
-        ret = fun()
-        if eval(expr):
-            return ret
-        time.sleep(0.001)
-    raise RuntimeError('timed out (ret=%r)' % ret)
-
-
-def retry_before_failing(ntimes=None):
-    """Decorator which runs a test function and retries N times before
-    actually failing.
-    """
-    def decorator(fun):
-        @functools.wraps(fun)
-        def wrapper(*args, **kwargs):
-            times = ntimes or NO_RETRIES
-            assert times, times
-            for x in range(times):
-                try:
-                    return fun(*args, **kwargs)
-                except AssertionError as _:
-                    err = _
-            raise err
-        return wrapper
-    return decorator
-
-
-def skip_on_access_denied(only_if=None):
-    """Decorator to Ignore AccessDenied exceptions."""
-    def decorator(fun):
-        @functools.wraps(fun)
-        def wrapper(*args, **kwargs):
-            try:
-                return fun(*args, **kwargs)
-            except psutil.AccessDenied:
-                if only_if is not None:
-                    if not only_if:
-                        raise
-                msg = "%r was skipped because it raised AccessDenied" \
-                      % fun.__name__
-                raise unittest.SkipTest(msg)
-        return wrapper
-    return decorator
-
-
-def skip_on_not_implemented(only_if=None):
-    """Decorator to Ignore NotImplementedError exceptions."""
-    def decorator(fun):
-        @functools.wraps(fun)
-        def wrapper(*args, **kwargs):
-            try:
-                return fun(*args, **kwargs)
-            except NotImplementedError:
-                if only_if is not None:
-                    if not only_if:
-                        raise
-                msg = "%r was skipped because it raised NotImplementedError" \
-                      % fun.__name__
-                raise unittest.SkipTest(msg)
-        return wrapper
-    return decorator
-
-
 def create_temp_executable_file(suffix, code="void main() { pause(); }"):
     tmpdir = None
     if TRAVIS and OSX:
@@ -557,21 +614,28 @@ def create_temp_executable_file(suffix, code="void main() { pause(); }"):
     return path
 
 
-if WINDOWS:
-    def get_winver():
-        wv = sys.getwindowsversion()
-        if hasattr(wv, 'service_pack_major'):  # python >= 2.7
-            sp = wv.service_pack_major or 0
-        else:
-            r = re.search("\s\d$", wv[4])
-            if r:
-                sp = int(r.group(0))
-            else:
-                sp = 0
-        return (wv[0], wv[1], sp)
-else:
-    def get_winver():
-        raise NotImplementedError("not a Windows OS")
+def cleanup():
+    reap_children(search_all=True)
+    safe_remove(TESTFN)
+    try:
+        safe_rmdir(TESTFN_UNICODE)
+    except UnicodeEncodeError:
+        pass
+    for path in _testfiles:
+        safe_remove(path)
+
+atexit.register(cleanup)
+atexit.register(lambda: DEVNULL.close())
+
+
+# ===================================================================
+# --- others
+# ===================================================================
+
+
+def warn(msg):
+    """Raise a warning msg."""
+    warnings.warn(msg, UserWarning)
 
 
 # In Python 3 paths are unicode objects by default.  Surrogate escapes
@@ -590,14 +654,3 @@ def decode_path(path):
                            errors="surrogateescape")
     else:
         return path
-
-
-def run_test_module_by_name(name):
-    # testmodules = [os.path.splitext(x)[0] for x in os.listdir(HERE)
-    #                if x.endswith('.py') and x.startswith('test_')]
-    name = os.path.splitext(os.path.basename(name))[0]
-    suite = unittest.TestSuite()
-    suite.addTest(unittest.defaultTestLoader.loadTestsFromName(name))
-    result = unittest.TextTestRunner(verbosity=VERBOSITY).run(suite)
-    success = result.wasSuccessful()
-    sys.exit(0 if success else 1)
