@@ -20,11 +20,6 @@ import textwrap
 import time
 import warnings
 
-try:
-    from unittest import mock  # py3
-except ImportError:
-    import mock  # requires "pip install mock"
-
 import psutil
 from psutil import LINUX
 from psutil._compat import PY3
@@ -33,10 +28,13 @@ from psutil.tests import call_until
 from psutil.tests import get_kernel_version
 from psutil.tests import importlib
 from psutil.tests import MEMORY_TOLERANCE
+from psutil.tests import mock
+from psutil.tests import PYPY
 from psutil.tests import pyrun
 from psutil.tests import reap_children
 from psutil.tests import retry_before_failing
 from psutil.tests import run_test_module_by_name
+from psutil.tests import safe_remove
 from psutil.tests import sh
 from psutil.tests import skip_on_not_implemented
 from psutil.tests import TESTFN
@@ -154,6 +152,13 @@ class TestSystemVirtualMemory(unittest.TestCase):
         self.assertAlmostEqual(cached, psutil.virtual_memory().cached,
                                delta=MEMORY_TOLERANCE)
 
+    @retry_before_failing()
+    @unittest.skipIf(TRAVIS, "fails on travis")
+    def test_shared(self):
+        total, used, free, shared = free_physmem()
+        self.assertAlmostEqual(shared, psutil.virtual_memory().shared,
+                               delta=MEMORY_TOLERANCE)
+
     # --- mocked tests
 
     def test_warnings_mocked(self):
@@ -166,8 +171,7 @@ class TestSystemVirtualMemory(unittest.TestCase):
                 w = ws[0]
                 self.assertTrue(w.filename.endswith('psutil/_pslinux.py'))
                 self.assertIn(
-                    "'cached', 'active' and 'inactive' memory stats couldn't "
-                    "be determined", str(w.message))
+                    "memory stats couldn't be determined", str(w.message))
                 self.assertEqual(ret.cached, 0)
                 self.assertEqual(ret.active, 0)
                 self.assertEqual(ret.inactive, 0)
@@ -331,6 +335,53 @@ class TestSystemNetwork(unittest.TestCase):
                     self.assertEqual(addr.address, get_ipv4_address(name))
                 # TODO: test for AF_INET6 family
 
+    def test_net_if_stats(self):
+        for name, stats in psutil.net_if_stats().items():
+            try:
+                out = sh("ifconfig %s" % name)
+            except RuntimeError:
+                pass
+            else:
+                self.assertEqual(stats.isup, 'RUNNING' in out)
+                self.assertEqual(stats.mtu,
+                                 int(re.findall('MTU:(\d+)', out)[0]))
+
+    def test_net_io_counters(self):
+        def ifconfig(nic):
+            ret = {}
+            out = sh("ifconfig %s" % name)
+            ret['packets_recv'] = int(re.findall('RX packets:(\d+)', out)[0])
+            ret['packets_sent'] = int(re.findall('TX packets:(\d+)', out)[0])
+            ret['errin'] = int(re.findall('errors:(\d+)', out)[0])
+            ret['errout'] = int(re.findall('errors:(\d+)', out)[1])
+            ret['dropin'] = int(re.findall('dropped:(\d+)', out)[0])
+            ret['dropout'] = int(re.findall('dropped:(\d+)', out)[1])
+            ret['bytes_recv'] = int(re.findall('RX bytes:(\d+)', out)[0])
+            ret['bytes_sent'] = int(re.findall('TX bytes:(\d+)', out)[0])
+            return ret
+
+        for name, stats in psutil.net_io_counters(pernic=True).items():
+            try:
+                ifconfig_ret = ifconfig(name)
+            except RuntimeError:
+                continue
+            self.assertAlmostEqual(
+                stats.bytes_recv, ifconfig_ret['bytes_recv'], delta=1024)
+            self.assertAlmostEqual(
+                stats.bytes_sent, ifconfig_ret['bytes_sent'], delta=1024)
+            self.assertAlmostEqual(
+                stats.packets_recv, ifconfig_ret['packets_recv'], delta=512)
+            self.assertAlmostEqual(
+                stats.packets_sent, ifconfig_ret['packets_sent'], delta=512)
+            self.assertAlmostEqual(
+                stats.errin, ifconfig_ret['errin'], delta=10)
+            self.assertAlmostEqual(
+                stats.errout, ifconfig_ret['errout'], delta=10)
+            self.assertAlmostEqual(
+                stats.dropin, ifconfig_ret['dropin'], delta=10)
+            self.assertAlmostEqual(
+                stats.dropout, ifconfig_ret['dropout'], delta=10)
+
     @unittest.skipUnless(which('ip'), "'ip' utility not available")
     @unittest.skipIf(TRAVIS, "skipped on Travis")
     def test_net_if_names(self):
@@ -652,18 +703,18 @@ class TestMisc(unittest.TestCase):
         # to 'localhost'.
         with mock.patch('psutil._pslinux.cext.users',
                         return_value=[('giampaolo', 'pts/2', ':0',
-                                      1436573184.0, True)]) as m:
+                                       1436573184.0, True)]) as m:
             self.assertEqual(psutil.users()[0].host, 'localhost')
             assert m.called
         with mock.patch('psutil._pslinux.cext.users',
                         return_value=[('giampaolo', 'pts/2', ':0.0',
-                                      1436573184.0, True)]) as m:
+                                       1436573184.0, True)]) as m:
             self.assertEqual(psutil.users()[0].host, 'localhost')
             assert m.called
         # ...otherwise it should be returned as-is
         with mock.patch('psutil._pslinux.cext.users',
                         return_value=[('giampaolo', 'pts/2', 'foo',
-                                      1436573184.0, True)]) as m:
+                                       1436573184.0, True)]) as m:
             self.assertEqual(psutil.users()[0].host, 'foo')
             assert m.called
 
@@ -686,6 +737,31 @@ class TestMisc(unittest.TestCase):
             psutil.PROCFS_PATH = "/proc"
             os.rmdir(tdir)
 
+    def test_sector_size_mock(self):
+        # Test SECTOR_SIZE fallback in case 'hw_sector_size' file
+        # does not exist.
+        def open_mock(name, *args, **kwargs):
+            if PY3 and isinstance(name, bytes):
+                name = name.decode()
+            if name.startswith("/sys/block/sda/queue/hw_sector_size"):
+                flag.append(None)
+                raise IOError(errno.ENOENT, '')
+            else:
+                return orig_open(name, *args, **kwargs)
+
+        flag = []
+        orig_open = open
+        patch_point = 'builtins.open' if PY3 else '__builtin__.open'
+        try:
+            with mock.patch(patch_point, side_effect=open_mock):
+                importlib.reload(psutil._pslinux)
+                importlib.reload(psutil)
+                self.assertEqual(flag, [None])
+                self.assertEqual(psutil._pslinux.SECTOR_SIZE, 512)
+        finally:
+            importlib.reload(psutil._pslinux)
+            importlib.reload(psutil)
+
 
 # =====================================================================
 # test process
@@ -694,6 +770,59 @@ class TestMisc(unittest.TestCase):
 @unittest.skipUnless(LINUX, "not a Linux system")
 class TestProcess(unittest.TestCase):
 
+    def setUp(self):
+        safe_remove(TESTFN)
+
+    tearDown = setUp
+
+    def test_compare_stat_and_status_files(self):
+        # /proc/pid/stat and /proc/pid/status have many values in common.
+        # Whenever possible, psutil uses /proc/pid/stat (it's faster).
+        # For all those cases we check that the value found in
+        # /proc/pid/stat (by psutil) matches the one found in
+        # /proc/pid/status.
+        for p in psutil.process_iter():
+            try:
+                f = psutil._psplatform.open_text('/proc/%s/status' % p.pid)
+            except IOError:
+                pass
+            else:
+                with f:
+                    for line in f:
+                        line = line.strip()
+                        if line.startswith('Name:'):
+                            name = line.split()[1]
+                            # Name is truncated to 15 chars
+                            self.assertEqual(p.name()[:15], name[:15])
+                        elif line.startswith('State:'):
+                            status = line[line.find('(') + 1:line.rfind(')')]
+                            status = status.replace(' ', '-')
+                            self.assertEqual(p.status(), status)
+                        elif line.startswith('PPid:'):
+                            ppid = int(line.split()[1])
+                            self.assertEqual(p.ppid(), ppid)
+                        # The ones below internally are determined by reading
+                        # 'status' file but we use a re to extract the info
+                        # so it makes sense to check them.
+                        elif line.startswith('Threads:'):
+                            num_threads = int(line.split()[1])
+                            self.assertEqual(p.num_threads(), num_threads)
+                        elif line.startswith('Uid:'):
+                            uids = tuple(map(int, line.split()[1:4]))
+                            self.assertEqual(tuple(p.uids()), uids)
+                        elif line.startswith('Gid:'):
+                            gids = tuple(map(int, line.split()[1:4]))
+                            self.assertEqual(tuple(p.gids()), gids)
+                        elif line.startswith('voluntary_ctxt_switches:'):
+                            vol = int(line.split()[1])
+                            self.assertAlmostEqual(
+                                p.num_ctx_switches().voluntary, vol, delta=2)
+                        elif line.startswith('nonvoluntary_ctxt_switches:'):
+                            invol = int(line.split()[1])
+                            self.assertAlmostEqual(
+                                p.num_ctx_switches().involuntary, invol,
+                                delta=2)
+
     def test_memory_maps(self):
         src = textwrap.dedent("""
             import time
@@ -733,12 +862,50 @@ class TestProcess(unittest.TestCase):
         time.sleep(.1)
         mem = p.memory_full_info()
         maps = p.memory_maps(grouped=False)
-        self.assertEqual(
-            mem.uss, sum([x.private_dirty + x.private_clean for x in maps]))
-        self.assertEqual(
-            mem.pss, sum([x.pss for x in maps]))
-        self.assertEqual(
-            mem.swap, sum([x.swap for x in maps]))
+        self.assertAlmostEqual(
+            mem.uss, sum([x.private_dirty + x.private_clean for x in maps]),
+            delta=4096)
+        self.assertAlmostEqual(
+            mem.pss, sum([x.pss for x in maps]), delta=4096)
+        self.assertAlmostEqual(
+            mem.swap, sum([x.swap for x in maps]), delta=4096)
+
+    # On PYPY file descriptors are not closed fast enough.
+    @unittest.skipIf(PYPY, "skipped on PYPY")
+    def test_open_files_mode(self):
+        def get_test_file():
+            p = psutil.Process()
+            giveup_at = time.time() + 2
+            while True:
+                for file in p.open_files():
+                    if file.path == os.path.abspath(TESTFN):
+                        return file
+                    elif time.time() > giveup_at:
+                        break
+            raise RuntimeError("timeout looking for test file")
+
+        #
+        with open(TESTFN, "w"):
+            self.assertEqual(get_test_file().mode, "w")
+        with open(TESTFN, "r"):
+            self.assertEqual(get_test_file().mode, "r")
+        with open(TESTFN, "a"):
+            self.assertEqual(get_test_file().mode, "a")
+        #
+        with open(TESTFN, "r+"):
+            self.assertEqual(get_test_file().mode, "r+")
+        with open(TESTFN, "w+"):
+            self.assertEqual(get_test_file().mode, "r+")
+        with open(TESTFN, "a+"):
+            self.assertEqual(get_test_file().mode, "a+")
+        # note: "x" bit is not supported
+        if PY3:
+            safe_remove(TESTFN)
+            with open(TESTFN, "x"):
+                self.assertEqual(get_test_file().mode, "w")
+            safe_remove(TESTFN)
+            with open(TESTFN, "x+"):
+                self.assertEqual(get_test_file().mode, "r+")
 
     def test_open_files_file_gone(self):
         # simulates a file which gets deleted during open_files()
@@ -763,45 +930,18 @@ class TestProcess(unittest.TestCase):
     # --- mocked tests
 
     def test_terminal_mocked(self):
-        with mock.patch('psutil._pslinux._psposix._get_terminal_map',
+        with mock.patch('psutil._pslinux._psposix.get_terminal_map',
                         return_value={}) as m:
             self.assertIsNone(psutil._pslinux.Process(os.getpid()).terminal())
             assert m.called
 
-    def test_num_ctx_switches_mocked(self):
-        with mock.patch('psutil._pslinux.open', create=True) as m:
-            self.assertRaises(
-                NotImplementedError,
-                psutil._pslinux.Process(os.getpid()).num_ctx_switches)
-            assert m.called
-
-    def test_num_threads_mocked(self):
-        with mock.patch('psutil._pslinux.open', create=True) as m:
-            self.assertRaises(
-                NotImplementedError,
-                psutil._pslinux.Process(os.getpid()).num_threads)
-            assert m.called
-
-    def test_ppid_mocked(self):
-        with mock.patch('psutil._pslinux.open', create=True) as m:
-            self.assertRaises(
-                NotImplementedError,
-                psutil._pslinux.Process(os.getpid()).ppid)
-            assert m.called
-
-    def test_uids_mocked(self):
-        with mock.patch('psutil._pslinux.open', create=True) as m:
-            self.assertRaises(
-                NotImplementedError,
-                psutil._pslinux.Process(os.getpid()).uids)
-            assert m.called
-
-    def test_gids_mocked(self):
-        with mock.patch('psutil._pslinux.open', create=True) as m:
-            self.assertRaises(
-                NotImplementedError,
-                psutil._pslinux.Process(os.getpid()).gids)
-            assert m.called
+    # TODO: re-enable this test.
+    # def test_num_ctx_switches_mocked(self):
+    #     with mock.patch('psutil._pslinux.open', create=True) as m:
+    #         self.assertRaises(
+    #             NotImplementedError,
+    #             psutil._pslinux.Process(os.getpid()).num_ctx_switches)
+    #         assert m.called
 
     def test_cmdline_mocked(self):
         # see: https://github.com/giampaolo/psutil/issues/639
